#include "ldap.h"

extern PRIVILEGE p_ldap;
extern LINUXCONF_SUBSYS subb;

/* 
	Master menu 
*/
void ldap_server_menu()
{
	const char *master_global = MSG_U(M_MASTER_GLOBAL,"Server");
	const char *master_database  = MSG_U(M_MASTER_DATABASE,"Database");

	static const char *tbopt[]={
		"", master_global,
		"", master_database,
		NULL
	};
		
	DIALOG_MENU dia;
	dia.new_menuitems(tbopt);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_MASTER_MENU,"LDAP server config")
			,MSG_U(I_MASTER_MENU
				,"Select server or database settings")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == master_global){
				ldap_server_config();
			}else if (key == master_database){
				ldap_db_select();
			}
		}
	}
}

/* 
	LDAP master control dialog
*/
void ldap_server_control()
{
	const char *server_start = MSG_U(M_SERVER_START,"Start LDAP server");
	const char *server_stop = MSG_U(M_SERVER_STOP,"Stop LDAP server");
	const char *server_restart = MSG_U(M_SERVER_RESTART,"Restart LDAP server");
	const char *server_status = MSG_U(M_SERVER_STATUS,"LDAP server status");	

	const char *tbopt[]={
		"", server_start,
 		"", server_stop,
		"", server_restart,
		"", server_status,
		NULL
	};
		
	DIALOG dia;
	dia.new_menuitems(tbopt);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_SERVER_CONTROL,"LDAP Server Control")
			,MSG_U(I_SERVER_CONTROL
				,"Here you can control the LDAP server")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == server_start){
				sys_command_title ("LDAP server","slapd.init","start");
			}else if (key == server_stop){
				sys_command_title ("LDAP server","slapd.init","stop");
			}else if (key == server_restart){
				sys_command_title ("LDAP server","slapd.init","restart");
			}else if (key == server_status){
				sys_command_title ("LDAP server status","ldapsearch","-s base -b cn=monitor objectclass=*");
			}
		}
	}
}

/* 
	LDAP system migrate
*/
void ldap_system_migrate()
{

	const char *key_ldap_migrate = MSG_U(M_MIGRATE,"LDAP system migrate");	

	DIALOG_MENU dia;
	dia.new_menuitem("",key_ldap_migrate);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_MIGRATE,"LDAP System Migration")
			,MSG_U(I_MIGRATE
				,"Use this to migrate system data into directory")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == key_ldap_migrate){
			  if (xconf_yesno(MSG_U(T_MIGRATE_OK,"LDAP database migration"),
					  MSG_U(I_MIGRATE_OK,
"This will migrate all system data\n\
from the localhost into the default LDAP database\n\
This is usually only done when installing.\n\
Are you sure you want to do this?\n"),
					  help_ldap) == MENU_OK)
			  {
			    sys_command_title ("LDAP db migrate","ldapmigrate","-y");
			  }
			}
		}
	}
}

/* 
	LDAP master config dialog
*/
void ldap_server_config()
{
	
VIEWITEMS v_slapd_conf;

/* Init config */
	// Global
	SSTRING slapd_schemacheck = "off";
	SSTRING slapd_loglevel = "0";
	SSTRING slapd_defaultaccess = "read";
	SSTRING slapd_referral = "";

/* Read config */
        v_slapd_conf.read (f_slapd_conf);
	slapd_schemacheck.setfrom (get_keyval(v_slapd_conf,"schemacheck"));
	slapd_loglevel.setfrom (get_keyval(v_slapd_conf,"loglevel"));
	slapd_defaultaccess.setfrom (get_keyval(v_slapd_conf,"defaultaccess"));
	slapd_referral.setfrom (get_keyval(v_slapd_conf,"referral"));
/* Draw dialog */
	DIALOG dia;
	dia.newf_title ("",MSG_U(I_MASTER_GLOBAL,"Global settings"));
	FIELD_COMBO *schema = dia.newf_combo(MSG_U(F_SLAPD_SCHEMACHECK,"Schemacheck"),slapd_schemacheck);
		schema->addopt ("on");
		schema->addopt ("off");
	dia.newf_str (MSG_U(F_SLAPD_LOGLEVEL,"Loglevel"),slapd_loglevel);
	FIELD_COMBO *accesslist = dia.newf_combo(MSG_U(F_SLAPD_DEFAULTACCESS,"Default access"),slapd_defaultaccess);
		accesslist->addopt ("none");
		accesslist->addopt ("compare");
		accesslist->addopt ("search");
		accesslist->addopt ("read");
		accesslist->addopt ("write");
		accesslist->addopt ("delete");
	dia.newf_str (MSG_U(F_SLAPD_REFERRAL,"Referral"),slapd_referral);
/* Wait for dialog */
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_MASTER_CONFIG,"LDAP master")
			,MSG_U(I_MASTER_CONFIG
				,"This is the LDAP master settings")				
			,help_ldap
			,nof);
/* Exit */
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
/* Save */
		else if (code == MENU_ACCEPT){
			set_keyval(v_slapd_conf,"schemacheck",slapd_schemacheck.get());
			set_keyval(v_slapd_conf,"loglevel",slapd_loglevel.get());
			set_keyval(v_slapd_conf,"defaultaccess",slapd_defaultaccess.get());
			set_keyval(v_slapd_conf,"referral",slapd_referral.get());
			v_slapd_conf.write (f_slapd_conf,&p_ldap);
			break;
		}
	}
}

/*
	Return list of all LDAP databases
*/
int ldap_db_getlist (SSTRINGS &lst)
{
	SSTRINGS tb;
	int n,i;				
	dir_getlist (LDAP_CONF_DIR,tb);
	n = tb.getnb();
	for (i=0; i<n; i++){
		if ((tb.getitem(i)->strstr(".db.")!=NULL)&&(tb.getitem(i)->strstr("~")==NULL)) {
			SSTRING *fname;
			fname = new SSTRING;
			fname->setfrom(tb.getitem(i)->get());
			lst.add(fname);
		}
	}
	lst.sort();
	tb.remove_all();
	return lst.getnb();
}

/*
	Select LDAP db
*/
void ldap_db_select()
{
	LDAPDB *db;
	SSTRINGS db_lst;
	DIALOG dia;
	int nof = 0;		
	dia.newf_head ("",MSG_U(F_DB_LIST,"LDAP DB list"));
		int n,i;
		db_lst.remove_all();	
		n = ldap_db_getlist(db_lst);
		for (i=0; i<n; i++){
				dia.new_menuitem (db_lst.getitem(i)->get(),"");
		}
	while (1){
			

		MENU_STATUS code = dia.editmenu (MSG_U(T_LDAP_DB_LIST,"LDAP database list")
				,MSG_U(I_LDAP_DB_LIST,"Select database")
				,help_ldap
				,nof
				,MENUBUT_ADD);
		if (code == MENU_QUIT || code == MENU_CANCEL || code == MENU_ESCAPE){					
			break;
		}
		else if (code == MENU_ADD){
			const char *name = "test.db.conf";
			db = new LDAPDB;
			db->config(name);
		}
		else if (nof >=0 && nof < db_lst.getnb()){
			const char *name = db_lst.getitem(nof)->get();
			db = new LDAPDB(name);
			db->menu();
		}
		dia.restore();	
	}
}

/* 
	LDAP Client config
	Used by normal user applications
*/

void ldap_client_config()
{

VIEWITEMS v_ldap_conf;

LDAPPROFILE profile;

/* Set config */
	SSTRING client_host = "localhost";
	SSTRING client_suffix = "dc=your,dc=domain";
	SSTRING client_binddn = "cn=root,dc=your,dc=domain";
	SSTRING client_bindpw = "secret";
	SSTRING client_scope = "sub";
	SSTRING client_port = "389";
/* Read conf */
	v_ldap_conf.read (f_ldap_conf);
	profile.host.setfrom (get_keyval(v_ldap_conf,"host"));
	profile.port.setfrom (get_keyval(v_ldap_conf,"port"));
	profile.base.setfrom (get_keyval(v_ldap_conf,"base"));
	profile.dn.setfrom (get_keyval(v_ldap_conf,"binddn"));
	profile.pw.setfrom (get_keyval(v_ldap_conf,"bindpw"));
	profile.scope.setfrom (get_keyval(v_ldap_conf,"scope"));
/* Draw dialog */
	DIALOG dia;
	dia.newf_str (MSG_R(F_API_HOST),profile.host);
	dia.newf_str (MSG_R(F_API_PORT),profile.port);
	dia.newf_str (MSG_R(F_API_BASE),profile.base);
	dia.newf_str (MSG_R(F_API_DN),profile.dn);
	dia.newf_str (MSG_R(F_API_PW),profile.pw);
	FIELD_COMBO *scopelist = dia.newf_combo(MSG_R(F_API_SCOPE),profile.scope);
		scopelist->addopt ("one");
		scopelist->addopt ("base");
		scopelist->addopt ("sub");
/* Wait for dialog */
	
	int nof = 0;

	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_CLIENT_MENU,"LDAP user client")
			,MSG_U(I_CLIENT_MENU
				,"This is the default binding for the user client.\nIt will be used by default\nwhen users run LDAP applications.")
				,help_ldap			
				,nof);
/* Exit */
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
/* Save */
		else if (code == MENU_ACCEPT){
			set_keyval(v_ldap_conf,"host",profile.host.get());
			set_keyval(v_ldap_conf,"port",profile.port.get());
			set_keyval(v_ldap_conf,"base",profile.base.get());
			set_keyval(v_ldap_conf,"binddn",profile.dn.get());
			set_keyval(v_ldap_conf,"bindpw",profile.pw.get());
			set_keyval(v_ldap_conf,"scope",profile.scope.get());
			v_ldap_conf.write (f_ldap_conf,&p_ldap);
			break;
		}
	}
}

/* 
	LDAP System client config
	Used by NSS/PAM LDAP
*/

void ldap_system_config()
{

VIEWITEMS v_ldap_sec;

LDAPPROFILE profile;

/* Set default */
	SSTRING client_host = "localhost";
	SSTRING client_port= "389";
	SSTRING client_suffix = "dc=your,dc=domain";
	SSTRING client_binddn = "cn=root,dc=your,dc=domain";
	SSTRING client_bindpw = "secret";
	SSTRING client_scope = "sub";
	SSTRING nss_crypt = "des";
	SSTRING pam_filter = "objectclass=account";
	SSTRING pam_login = "uid";
	SSTRING pam_lookup = "no";
	SSTRING pam_group = "cn=PAM,ou=Groups,dc=your,dc=domain";
	SSTRING pam_member = "uniquemember";
	SSTRING pam_crypt = "local";

/* Read conf */
	v_ldap_sec.read (f_ldap_sec);
	profile.host.setfrom (get_keyval(v_ldap_sec,"host"));
	profile.port.setfrom (get_keyval(v_ldap_sec,"port"));
	profile.base.setfrom (get_keyval(v_ldap_sec,"base"));
	profile.dn.setfrom (get_keyval(v_ldap_sec,"binddn"));
	profile.pw.setfrom (get_keyval(v_ldap_sec,"bindpw"));
	profile.scope.setfrom (get_keyval(v_ldap_sec,"scope"));
	nss_crypt.setfrom (get_keyval(v_ldap_sec,"crypt"));
	pam_filter.setfrom (get_keyval(v_ldap_sec,"pam_filter"));
	pam_login.setfrom (get_keyval(v_ldap_sec,"pam_login_attribute"));
	pam_lookup.setfrom (get_keyval(v_ldap_sec,"pam_lookup_policy"));
	pam_group.setfrom (get_keyval(v_ldap_sec,"pam_groupdn"));
	pam_member.setfrom (get_keyval(v_ldap_sec,"pam_member_attribute"));
	pam_crypt.setfrom (get_keyval(v_ldap_sec,"pam_crypt"));
	
/* Draw dialog */
	DIALOG dia;
	dia.newf_title ("",MSG_R(I_LDAPBIND));
	dia.newf_str (MSG_R(F_API_HOST),profile.host);
	dia.newf_str (MSG_R(F_API_PORT),profile.port);
	dia.newf_str (MSG_R(F_API_BASE),profile.base);
	dia.newf_str (MSG_R(F_API_DN),profile.dn);
	dia.newf_str (MSG_R(F_API_PW),profile.pw);
	FIELD_COMBO *scopelist = dia.newf_combo(MSG_R(F_API_SCOPE),profile.scope);
		scopelist->addopt ("one");
		scopelist->addopt ("base");
		scopelist->addopt ("sub");
	dia.newf_title ("",MSG_U(I_NSS_OPTIONS,"NSS options"));
	FIELD_COMBO *cryptlist = dia.newf_combo(MSG_U(F_NSS_CRYPT,"libc hash"),nss_crypt);
		cryptlist->addopt ("des");
		cryptlist->addopt ("sha");
		cryptlist->addopt ("md5");
	dia.newf_title ("",MSG_U(I_PAM_OPTIONS,"PAM options"));
	dia.newf_str (MSG_U(F_PAM_FILTER,"PAM filter"),pam_filter);
	dia.newf_str (MSG_U(F_PAM_LOGIN,"PAM login attr"),pam_login);
	dia.newf_str (MSG_U(F_PAM_LOOKUP,"PAM lookup policy"),pam_lookup);
	dia.newf_str (MSG_U(F_PAM_GRUOP,"PAM group"),pam_group);
	dia.newf_str (MSG_U(F_PAM_MEMBER,"PAM member attr"),pam_member);
	dia.newf_str (MSG_U(F_PAM_CRYPT,"PAM crypt"),pam_crypt);
/* Wait for dialog */
	
	int nof = 0;

	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_SYSTEM_MENU,"LDAP system client")
			,MSG_U(I_SYSTEM_MENU
				,"This is the binding for the system client.\nThey will be used by the system\nif you enable the LDAP NSS/PAM system.\nBe careful when configuring this.")				
			,help_ldap
			,nof);
/* Exit */
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
/* Save */
		else if (code == MENU_ACCEPT){
			set_keyval(v_ldap_sec,"host",profile.host.get());
			set_keyval(v_ldap_sec,"port",profile.port.get());
			set_keyval(v_ldap_sec,"base",profile.base.get());
			set_keyval(v_ldap_sec,"binddn",profile.dn.get());
			set_keyval(v_ldap_sec,"bindpw",profile.pw.get());
			set_keyval(v_ldap_sec,"scope",profile.scope.get());
			set_keyval(v_ldap_sec,"crypt",nss_crypt.get());
	 		set_keyval(v_ldap_sec,"pam_filter",pam_filter.get());
	 		set_keyval(v_ldap_sec,"pam_login_attribute",pam_login.get());
	 		set_keyval(v_ldap_sec,"pam_lookup_policy",pam_lookup.get());
	 		set_keyval(v_ldap_sec,"pam_groupdn",pam_group.get());
	 		set_keyval(v_ldap_sec,"pam_member_attribute",pam_member.get());
	 		set_keyval(v_ldap_sec,"pam_crypt",pam_crypt.get());			
			v_ldap_sec.write (f_ldap_sec,&p_ldap);
			break;
		}
	}
}



