/*********** 

LDAPCONF

Licence: Linuxconf/GPL (see the file COPYING for details)
Author: Stein Vrle <stein@terminator.net>

************/

#include "ldap.h"

/*
  LDAPOBJECT
  This object holds all information about one LDAP object, retrived from an LDAP directory.
  It provides functions to manipulate and retrive information in this object,
  and methods to syncronize the object with the LDAP database server.
*/
PUBLIC LDAPOBJECT::LDAPOBJECT ()
{
  filter = "";
  base = "";
	binddn = "";
	bindpw = "";
	host = "";
	ldif_format = 0;
}



/*

  LDAPOBJECT database methods

*/
/*
	LDAPOBJECT 
	Add to database
*/
PUBLIC int LDAPOBJECT::add ()
{
int ret=0;
return ret;
}

/*
	LDAPOBJECT
	Update to database
*/
PUBLIC int LDAPOBJECT::modify ()
{
int ret=0;

	const char *args;
	SSTRING parameters;
	SSTRING dataentry;
	SSTRINGS resmsg;

	/* Build dataentry */
        FILE *fout = fopen ("/tmp/ldapconf.entry","w");
        if (fout != NULL){
		fprintf (fout,"%s\n",dn.get());
		for (int i=0; i<attributes.getnb(); i++){
		  	fprintf (fout,"%s\n",attributes.getitem(i)->get());
		}
		for (int i=0; i<objectclasses.getnb(); i++){
			fprintf (fout,"%s\n",objectclasses.getitem(i)->get());
		}
                fclose (fout);
        }

	/* Build commandline */
	if ( ! base.is_empty() ) {parameters.append(" -b "); parameters.append(base.get());}
	if ( ! binddn.is_empty() ) {parameters.append(" -D "); parameters.append(binddn.get());}
	if ( ! bindpw.is_empty() ) {parameters.append(" -w "); parameters.append(bindpw.get());}
	if ( ! host.is_empty() ) {parameters.append(" -h "); parameters.append(host.get());}
	if ( ! port.is_empty() ) {parameters.append(" -p "); parameters.append(port.get());}
	parameters.append(" -r ");
	parameters.append("-f /tmp/ldapconf.entry");
	args = parameters.get();

	/* Run command*/
	sys_command ("ldapmodify",args,resmsg);

	// unlink("/tmp/ldapconf.entry");

	// xconf_notice(args);

	/* Show result */
	dialog_textbox("LDAP Modify Result",resmsg);

return ret;
}

/*
	LDAPOBJECT
	Delete from database
*/
PUBLIC int LDAPOBJECT::del ()
{
int ret=0;
return ret;
}

/*
	LDAPOBJECT search
	Search uniq object
*/
PUBLIC int LDAPOBJECT::search ()
{
	int ret=0;
	const char *args;
	SSTRING parameters;
	SSTRINGS data;
	
	/* Build commandline */
	if ( ! base.is_empty() ) {parameters.append(" -b "); parameters.append(base.get());}
	if ( ! binddn.is_empty() ) {parameters.append(" -D "); parameters.append(binddn.get());}
	if ( ! bindpw.is_empty() ) {parameters.append(" -w "); parameters.append(bindpw.get());}
	if ( ! host.is_empty() ) {parameters.append(" -h "); parameters.append(host.get());}
	if ( ! port.is_empty() ) {parameters.append(" -p "); parameters.append(port.get());}
	if (ldif_format) {parameters.append(" -L ");}
	if ( ! filter.is_empty() ) {parameters.append(" "); parameters.append(filter.get());}
	if ( ! attr.is_empty() ) {parameters.append(" "); parameters.append(attr.get());}
	args = parameters.get();

	/* Run ldapsearch command*/
	ret = sys_command ("ldapsearch",args,data);

	/* Empty result? */

	/* Split the objects */

	int c = 1; 	/* Line counter */ 
	int d;		/* Number of lines */
	int idx = 0;	/* Object index number */	
	int i = 1; /* Object internal count */
	char *a;
	char *c1;	
	SSTRING *s = NULL;
	
	d = data.getnb();
	c = 0;
	c++; /* Skip command message */

	while (c < d ) {
			s = new SSTRING;
			s->setfrom(data.getitem(c)->get());
			a = s->get();
			a = str_skip(a);
			c1 = strip_end(a);
//			s->setfrom(a);			
			
			if ( s->getlen() == 4 ) {		
				i = 1;		
			} else if (i==1) { /* first line is dn */
				dn.setfrom(a);
				idx++;
				i = 2;
			} else if (strncmp(a,"objectclass",11) == 0){ 
				oc_add(a);
		 	} else {
				s->setfrom(a); 
				attributes.add(s);
			}
			c++;
	}
	idx++;
	return idx;
}

/*
	LDAPOBJECT
	Search and return resultlist
*/
PUBLIC int LDAPOBJECT::search_list (SSTRINGS&lst)
{
	int ret=0;
	const char *args;
	SSTRING parameters;
	SSTRINGS data;
	
	/* Build commandline */
	if ( ! base.is_empty() ) {parameters.append(" -b "); parameters.append(base.get());}
	if ( ! binddn.is_empty() ) {parameters.append(" -D "); parameters.append(binddn.get());}
	if ( ! bindpw.is_empty() ) {parameters.append(" -w "); parameters.append(bindpw.get());}
	if ( ! host.is_empty() ) {parameters.append(" -h "); parameters.append(host.get());}
	if (ldif_format) {parameters.append(" -L ");}
	if ( ! filter.is_empty() ) {parameters.append(" "); parameters.append(filter.get());}
	parameters.append(" uid");
	args = parameters.get();

	/* Run ldapsearch command*/
	ret = sys_command ("ldapsearch",args,data);

	/* Split the objects */

	int c=0; 	/* Line counter */ 
	int d;		/* Number of lines */
	int idx = 0;	/* Object index number */	
	int i = 1; /* Object internal count */
	char *a;
	char *c1;	
	SSTRING *s = NULL;
      
	d = data.getnb();

	while (c < d ) {
			s = new SSTRING;
			s->setfrom(data.getitem(c)->get());
			a = s->get();
			a = str_skip(a);
			c1 = strip_end(a);					
			if ( s->getlen() == 4 ) {		
				i = 1;	
			} else if (i==1) {/* first line is dn */
				idx++;
				i = 2;
			} else if (i==2) {/* second line is uid */
				s->setfrom(a);	
				lst.add(s);
				i = 3;
			}
	c++;
	}
	idx++;
	return idx;
}

/*
	LDAPOBJECT objectclass delete
	Delete objectclass
*/
PUBLIC int LDAPOBJECT::oc_del (const char *oc)
{
int ret=0;
return ret;
}

/*
  LDAPOBJECT datamanipulation
*/

/*
	LDAPOBJECT
	List all attributes keys.
	Return number of attributes.
*/
PUBLIC int LDAPOBJECT::at_list (SSTRINGS&lst)
{
  int n = attributes.getnb();

  return n;
}

/*
	LDAPOBJECT
	Del one attribute
*/

PUBLIC int LDAPOBJECT::at_del (const char *at_key)
{
	int ret=0;
	return ret;
}

/*
	Locate a VIEWITEM which is an assignment of the variable var
	(var=value)
	(from linuxconf fview.cc)
*/
PUBLIC SSTRING* LDAPOBJECT::locateassign (const char *var)
{
	int lenvar = strlen(var);
	SSTRING  *ret = NULL;
	int n = attributes.getnb();
	for (int i=0; i<n; i++){
	        SSTRING *it = attributes.getitem (i);
		const char *pt = it->get();
		pt = str_skip(pt);
		if (strncmp(pt,var,lenvar)==0 && pt[lenvar] == '='){
			ret = it;
			break;
		}
	}
	return ret;
}


/*
	LDAPOBJECT	
	Get one attribute
*/
PUBLIC char* LDAPOBJECT::at_get (const char *var)
{

/*
  (Taken from linuxconf fviews.cc )
  Locate the value of a variable.
  Return NULL if the variable is not defined

	The quote surrounding the value are removed.
*/


  char tmp[1000];
	const char *ret = NULL;
	SSTRING *it = locateassign(var);
	if (it != NULL){
		const char *pt = it->strchr('=');
		if (pt != NULL){
			ret = pt+1;
			if (ret[0] == '"'){
				ret++;
				strcpy (tmp,ret);
				strip_end (tmp);
				int len = strlen(tmp)-1;
				if (len >= 0 && tmp[len] == '"') tmp[len] = '\0';
				ret = tmp;
			}
		}
	}
	return ret;

}

/*
	Update or add an assignement of the form var="val"
*/
PUBLIC void LDAPOBJECT::at_set (const char *var ,const char *val)
{
	if (strcmp(val,"")!=0) {	
		SSTRING *it = locateassign(var);
		if (it == NULL){
			it = new SSTRING("");
			attributes.add (it);
		}
		char buf[1000];
		snprintf (buf,sizeof(buf)-1,"%s=%s",var,val);
		it->setfrom (buf);
	}
}

/*
	LDAPOBJECT
	List objeclasses
*/
PUBLIC int LDAPOBJECT::oc_list (SSTRINGS&lst)
{
int ret=0;
return ret;
}

/*
	LDAPOBJECT
	Add objectclass
*/
PUBLIC int LDAPOBJECT::oc_add (const char *oc)
{
	int ret=0;
	SSTRING *s;
	s = new SSTRING;
	s->setfrom(oc);
	objectclasses.add(s);
	return ret;
}
