/*********** 

LDAPCONF

Licence: Linuxconf/GPL (see the file COPYING for details)
Author: Stein Vrle <stein@terminator.net>

************/

#pragma implementation
#include "ldap.h"

HELP_FILE help_ldap ("ldapconf","ldap");

LINUXCONF_SUBSYS subb (subsys_ldap,P_MSG_U(M_LDAP_SUBSYS,"LDAP system"));
PRIVILEGE p_ldap (subsys_ldap,P_MSG_R(M_LDAP_SUBSYS),P_MSG_U(T_PMISC,"9-Miscellaneous"));
PRIVILEGE p_ldap_user (subsys_ldap,P_MSG_R(M_LDAP_SUBSYS),P_MSG_R(T_PMISC));


CONFIG_FILE f_slapd_conf ("/etc/ldap/slapd.conf",help_ldap
		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
		,"root","root",0400
		,subsys_ldap);
CONFIG_FILE f_ldap_conf ("/etc/ldap/ldap.conf",help_ldap
		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
		,subsys_ldap);
CONFIG_FILE f_ldap_sec ("/etc/ldap/ldap.sec",help_ldap
		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
		,"root","root",0400
		,subsys_ldap);
CONFIG_FILE f_nss_conf ("/etc/nsswitch.conf",help_ldap
		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
		,subsys_ldap);

MODULE_DEFINE_VERSION(ldapconf);

PUBLIC MODULE_ldapconf::MODULE_ldapconf()
	: LINUXCONF_MODULE("ldapconf")
{
	linuxconf_loadmsg ("ldapconf",PACKAGE_REV);
}

static const char *key_ldap_main = NULL;
static const char *key_ldap_mainmenu_client = NULL;
static const char *key_ldap_client_config = NULL;
static const char *key_ldap_client = NULL;
static const char *key_ldap_mainmenu_server = NULL;
static const char *key_ldap_server_config = NULL;
static const char *key_ldap_server_control = NULL;
static const char *key_ldap_db_config = NULL;
static const char *key_ldap_mainmenu_system = NULL;
static const char *key_ldap_system_config = NULL;
static const char *key_pam_config = NULL;
static const char *key_nss_config = NULL;
static const char *menu_status = NULL;

PUBLIC void MODULE_ldapconf::setmenu (
	DIALOG &dia,
	MENU_CONTEXT context)
{
	if (context == MENU_MISCSERV){
		key_ldap_main = MSG_U(M_LDAP_MAIN,"LDAP directory service");
		dia.new_menuitem ("ldapconf","",key_ldap_main);
	}
	if (context == MENU_NETWORK_CLIENT){
		key_ldap_client_config = MSG_U(M_LDAP_CLIENT_CONFIG,"LDAP Client");
		dia.new_menuitem ("ldapconf","",key_ldap_client_config);
	}
	if (context == MENU_NETWORK_SERVER){
		key_ldap_server_config = MSG_U(M_LDAP_SERVER_CONFIG,"LDAP Server");
		dia.new_menuitem ("ldapconf","",key_ldap_server_config);
	}
	if (context == MENU_CONTROL_PANEL){
		key_ldap_server_control = MSG_U(M_LDAP_SERVER_CONTROL,"LDAP Control");
		dia.new_menuitem ("ldapconf","",key_ldap_server_control);
	} 
	if (context == MENU_SYS_STATUS){
		menu_status = MSG_U(M_LDAP_STATUS,"LDAP server status");
		dia.new_menuitem ("ldapconf","",menu_status);
	}
}

PUBLIC int MODULE_ldapconf::domenu (
	MENU_CONTEXT context,
	const char *key)
{
	if (context == MENU_MISCSERV){
		if (key == key_ldap_main){
			ldap_mainmenu();
		}
	} else 
	if (context == MENU_NETWORK_CLIENT){
		if (key == key_ldap_client_config){
			ldap_client_config();
		}
	} else 
	if (context == MENU_NETWORK_SERVER){
		if (key == key_ldap_server_config){
			ldap_server_menu();
		}
	} else 
	if (context == MENU_CONTROL_PANEL){
		if (key == key_ldap_server_control){
			ldap_server_control();
		}
	} else 
	if (context == MENU_SYS_STATUS){
		if (key == menu_status){
			ldap_status();
		}
	}
	return 0;
}

PUBLIC int MODULE_ldapconf::dohtml (const char *key)
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(key,"ldapconf")==0){
		ldap_client_menu();
		ret = 0;
	}
	return ret;
}

static void usage()
{
	xconf_error (MSG_U(T_USAGE
		,"linuxconf --modulemain ldapconf usage\n"
		 "\n"
		 "    ldapconf --option ...\n")
		);
}

PUBLIC void MODULE_ldapconf::usage (SSTRINGS &tb)
{
	tb.add (new SSTRING (MSG_R(T_USAGE)));
}

PUBLIC int MODULE_ldapconf::execmain (int argc , char *argv[])
{
	int ret = LNCF_NOT_APPLICABLE;
	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}
	if (strcmp(pt,"ldapconf")==0){
		ret = -1;
		if (argc == 1){
			ldap_mainmenu();
		}else{
			// ### Add some option parsing for the module
			::usage();
		}
	}
	return ret;
}

PUBLIC int MODULE_ldapconf::message (
        const char *msg,        // message
        int,                    // argc
        const char *[])         // argv[]
{
        int ret = LNCF_NOT_APPLICABLE;
        if (strcmp(msg,"listspc")==0){
                ret = 0;
		html_printf ("<hr>\n");
                html_printf ("<center><a href=/htmlmod:ldapconf:>%s"
                                ,MSG_U(T_HTMLMENU,"LDAP tools"));
                html_printf ("</A></center><P>\n");
        }	
        return ret;
}

static MODULE_ldapconf ldapconf;

/* 
	LDAPCONF Main Menu
	This is all the available submenus
	They will be linked into various places in the Linuxconf menutree,
	but will also be available in one main menu.
	Create a symlink from linuxconf to ldapconf and this main menu will show up.
*/

void ldap_mainmenu()
{
	
	key_ldap_mainmenu_server = MSG_U(M_LDAP_MAINMENU_SERVER,"LDAP Server");
	key_ldap_mainmenu_client = MSG_U(M_LDAP_MAINMENU_CLIENT,"LDAP Client");
	key_ldap_mainmenu_system = MSG_U(M_LDAP_MAINMENU_SYSTEM,"LDAP System");

	const char *tbopt[]={
		"", key_ldap_mainmenu_client,	
		"", key_ldap_mainmenu_server,	
		"", key_ldap_mainmenu_system,
		NULL
	};
		
	DIALOG_MENU dia;
	dia.new_menuitems(tbopt);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_LDAP_MENU,"LDAP")
			,MSG_U(I_LDAP_MENU
				,"This is the main parts of LDAPconf.")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == key_ldap_mainmenu_server){
				ldap_mainmenu_server();
			}else if (key == key_ldap_mainmenu_client){
				ldap_mainmenu_client();
			}else if (key == key_ldap_mainmenu_system){
				ldap_mainmenu_system();
			}
		}
	}
}

/* 
	LDAP mainmenu - client
*/
void ldap_mainmenu_client()
{  
	key_ldap_client = MSG_U(M_LDAP_CLIENT_TOOLS,"LDAP client tools");
	key_ldap_client_config = MSG_U(M_LDAP_MAIN_CLIENT_CONFIG,"Config LDAP client");
    
	DIALOG_MENU dia;
	dia.new_menuitem("",key_ldap_client);
	dia.new_menuitem("",key_ldap_client_config);
  	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_MAINMENU_CLIENT,"LDAP client menu")
						 ,MSG_U(I_MAINMENU_CLIENT,"This is for client mode of LDAP")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == key_ldap_client_config){
				ldap_client_config();
			}else if (key == key_ldap_client){
				ldap_client_menu();		       
			}
		}
	}
}

/* 
	LDAP mainmenu - server
*/
void ldap_mainmenu_server()
{  
	key_ldap_server_control = MSG_U(M_LDAP_MAIN_SERVER_CONTROL,"Control LDAP Server");
	key_ldap_server_config = MSG_U(M_LDAP_MAIN_SERVER_CONFIG,"Config LDAP Server");
	key_ldap_db_config = MSG_U(M_LDAP_DB_CONFIG,"Config LDAP Database");

	DIALOG_MENU dia;
	dia.new_menuitem("",key_ldap_server_control);
	dia.new_menuitem("",key_ldap_server_config);
	dia.new_menuitem("",key_ldap_db_config);

	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_MAINMENU_SERVER,"LDAP server menu")
						 ,MSG_U(I_MAINMENU_SERVER,"This menus are for running a LDAP server.")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == key_ldap_server_config){
				ldap_server_config();
			}else if (key == key_ldap_server_control){
				ldap_server_control();			
			}else if (key == key_ldap_db_config){
				ldap_db_select();
			}
		}
	}
}


/* 
	LDAP mainmenu - system
*/
void ldap_mainmenu_system()
{	
  	key_ldap_system_config = MSG_U(M_LDAP_SYSTEM_CONFIG,"Config LDAP System");
	key_pam_config = MSG_U(M_PAM_CONFIG,"Config PAM");
	key_nss_config = MSG_U(M_NSS_CONFIG,"Config NSS");

	DIALOG_MENU dia;
	dia.new_menuitem("",key_ldap_system_config);
	dia.new_menuitem("",key_pam_config);
	dia.new_menuitem("",key_nss_config);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_MAINMENU_SYSTEM,"LDAP system menu")
						 ,MSG_U(I_MAINMENU_SYSTEM,"This menus are for using LDAP NSS and PAM.\nWarning: Do not save in this menus, unless you now what you are doing.")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);

			if (key == key_ldap_system_config){
				ldap_system_config();
			}else if (key == key_pam_config){
				pam_config_main();
			}else if (key == key_nss_config){
				nss_config();
			}
		}
	}
}































